'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (req, res, option, server) {
    var mockFilename = matchMock(req, option);

    if (!mockFilename && option.target) {
        return proxyRequest(req, res, option, server);
    } else if (mockFilename) {
        var responseJson = null;
        try {
            responseJson = require(mockFilename) || require(mockFilename).default;
        } catch (e) {
            responseJson = 'mock file dest: ' + mockFilename + ' is not exsited';
            console.error(responseJson);
        }

        res.setHeader('Content-type', 'Application/json');
        res.write(JSON.stringify(responseJson));
        return res.end();
    }

    res.writeHead(404, { 'Content-type': 'text/html; charset=utf-8' });
    res.end('option.target is expected a url but got ' + option.target);
};

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _path = require('path');

var _fs = require('fs');

var _common = require('./common');

var _os = require('os');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function matchMock(req, option) {
    var relUrl = req.url;
    var mock = option.mock;
    var mockJsonLoc = false;
    if (mock && mock.rules && mock.rules !== 0) {
        mock.rules.every(function (rule) {
            var from = rule.from;
            if (relUrl.indexOf(from) === 0) {
                mockJsonLoc = rule.to + relUrl.substring(rule.from.length, relUrl.length);
                return false;
            }

            return true;
        });
    }
    return mockJsonLoc ? (0, _path.resolve)(mock.base, mockJsonLoc) : false;
}

function proxyRequest(req, res, option, server) {
    var requestOption = parseRequestOption(req, option);
    var request = requestOption.protocol === 'https:' ? _https2.default.request : _http2.default.request;

    var proxyReq = request(requestOption, function (proxyRes) {
        server.emit('proxyRes', { res: res, req: req, proxyRes: proxyRes, option: option });
        writeHeaders(res, proxyRes);
        proxyRes.pipe(res);
    });

    proxyReq.on('error', function (e) {
        if (e.code === 'ETIMEDOUT') {
            console.error(e.message);
        }
        server.emit('error', { e: e, req: req, res: res });
    });

    req.on('error', function (e) {
        console.log(e);
    });

    proxyReq.end();
}

function parseRequestOption(req, option) {
    var path = req.url;
    var _option$target = option.target,
        host = _option$target.host,
        hostname = _option$target.hostname,
        protocol = _option$target.protocol,
        port = _option$target.port;
    var headers = req.headers,
        method = req.method;

    headers.host = hostname;

    if (option.headers) {
        headers = Object.assign({}, headers, option.headers);
    }

    if (option.method) {
        method = option.method;
    }

    return Object.assign({}, { host: host, hostname: hostname, protocol: protocol, port: port, path: path }, { method: method, headers: headers });
}

function writeHeaders(res, proxyRes) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = Object.entries(proxyRes.headers)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _ref = _step.value;

            var _ref2 = _slicedToArray(_ref, 2);

            var key = _ref2[0];
            var header = _ref2[1];

            res.setHeader(key, header);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    var statusCode = proxyRes.statusCode,
        statusMessage = proxyRes.statusMessage;

    res.writeHead(statusCode, statusMessage);
}